-----------------------------------------------------------------------------------------------
-- Client Lua Script for CurrencyCount
-- Shows all currencies next to the gold display
-- Made by Jerodar (jerodar@enigmaguild.org)
-- Version 1.9 - 2017/03/20
-- Copyright 2015 Jeroen Oostdam
-----------------------------------------------------------------------------------------------
-- Changelog for 1.10:
--    - Added all of the primal essences (Thanks again Zodbrain!)
----------------------------------------------------------------------------------------------- 

require "Window"
 
local CurrencyCount = {}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CurrencyCount:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self 

  o.tCurrencyDisplays  = {}
  o.timerInit = nil

  o.tOptions = {}
  o.tOptions.bShowPrestige = true
  o.tOptions.bShowRenown = true
  o.tOptions.bShowCrafting = true
  o.tOptions.bShowElder = false
  o.tOptions.bShowGlory = true
  o.tOptions.bShowTriploons = false
  o.tOptions.bShowOmni = true
  o.tOptions.bShowService = true
  o.tOptions.bShowFortune = true
  o.tOptions.bShowNCoin = true
  o.tOptions.bShowPromisorry = false
  o.tOptions.bShowCredits = false
  o.tOptions.bSaticWidth = false
  o.tOptions.nSetWidth = 70
  o.tOptions.bEnableBackground = false

  o.nCurrentWidth = 0

  -- hardcoded margin between currency displays
  o.nMargin = 5

  o.karCurrency =
  {
    {eType = Money.CodeEnumCurrencyType.Credits,
      strTitle = "Money",
      strDescription = ""},
    {eType = Money.CodeEnumCurrencyType.Renown,
      strTitle = Apollo.GetString("CRB_Renown"),
      strDescription = Apollo.GetString("CRB_Renown_Desc")},
    {eType = Money.CodeEnumCurrencyType.ElderGems,
      strTitle = Apollo.GetString("CRB_Elder_Gems"),
      strDescription = Apollo.GetString("CRB_Elder_Gems_Desc")},
    {eType = Money.CodeEnumCurrencyType.Glory,
      strTitle = Apollo.GetString("CRB_Glory"),
      strDescription = Apollo.GetString("CRB_Glory_Desc")},
    {eType = Money.CodeEnumCurrencyType.Prestige,
      strTitle = Apollo.GetString("CRB_Prestige"),
      strDescription = Apollo.GetString("CRB_Prestige_Desc")},
    {eType = Money.CodeEnumCurrencyType.CraftingVouchers,
      strTitle = Apollo.GetString("CRB_Crafting_Vouchers"),
      strDescription = Apollo.GetString("CRB_Crafting_Voucher_Desc")},
    {eType = AccountItemLib.CodeEnumAccountCurrency.Omnibits,
      strTitle = Apollo.GetString("CRB_OmniBits"),
      strDescription = Apollo.GetString("CRB_OmniBits_Desc"),
      bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.ServiceToken,
      strTitle = Apollo.GetString("AccountInventory_ServiceToken"),
      strDescription = Apollo.GetString("AccountInventory_ServiceToken_Desc"),
      bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.MysticShiny,
      strTitle = Apollo.GetString("CRB_FortuneCoin"),
      strDescription = Apollo.GetString("CRB_FortuneCoin_Desc"),
      bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.NCoins,
      strTitle = Apollo.GetString("AccountInventory_NCoin"),
      strDescription = Apollo.GetString("AccountInventory_NCoin_Desc"),
      bAccountItem = true},
    {eType = AccountItemLib.CodeEnumAccountCurrency.PromissoryNote,
      strTitle = Apollo.GetString("CRB_Protostar_Promissory_Note"),
      strDescription = Apollo.GetString("CRB_Protostar_Promissory_Note_Desc"),
      bAccountItem = true},
    {eType = Money.CodeEnumCurrencyType.Triploons,
      strTitle = Apollo.GetString("CRB_Triploons"),
      strDescription = Apollo.GetString("CRB_Triploons_Desc")},
    {eType = Money.CodeEnumCurrencyType.PurpleEssence,
      strTitle = Apollo.GetString("Matrix_NodePurpleName"),
      strDescription = AccountItemLib.GetAccountCurrency(AccountItemLib.CodeEnumAccountCurrency.PurpleEssence):GetTypeString()},
    {eType = Money.CodeEnumCurrencyType.RedEssence,
      strTitle = Apollo.GetString("Matrix_NodeRedName"),
      strDescription = AccountItemLib.GetAccountCurrency(AccountItemLib.CodeEnumAccountCurrency.RedEssence):GetTypeString()},
    {eType = Money.CodeEnumCurrencyType.BlueEssence,
      strTitle = Apollo.GetString("Matrix_NodeBlueName"),
      strDescription = AccountItemLib.GetAccountCurrency(AccountItemLib.CodeEnumAccountCurrency.BlueEssence):GetTypeString()},
    {eType = Money.CodeEnumCurrencyType.GreenEssence,
      strTitle = Apollo.GetString("Matrix_NodeGreenName"),
      strDescription = AccountItemLib.GetAccountCurrency(AccountItemLib.CodeEnumAccountCurrency.GreenEssence):GetTypeString()}
  }

  return o
end

function CurrencyCount:Init()
  local bHasConfigureFunction = true
  local strConfigureButtonText = "Currency Count"
  local tDependencies = {
    -- "UnitOrPackageName",
  }
  Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function CurrencyCount:OnSave(eLevel)
  if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  local tData = {}

  for key, value in pairs(self.tOptions) do
    tData[key] = value
  end

  return tData
end

function CurrencyCount:OnRestore(eLevel, tData)
  for key, value in pairs(tData) do
    self.tOptions[key] = value
  end
end

-----------------------------------------------------------------------------------------------
-- CurrencyCount OnLoad
-----------------------------------------------------------------------------------------------
function CurrencyCount:OnLoad()
  -- load our form file
  self.xmlDoc = XmlDoc.CreateFromFile("CurrencyCount.xml")
  self.xmlDoc:RegisterCallback("OnDocumentReady", self)
end

function CurrencyCount:OnDocumentReady()

  if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CurrencyList", nil, self)
    
    if self.wndMain == nil then
      Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
      return
    end

    self.wndMain:Show(true, true)
    -- Register for currency update events
    Apollo.RegisterEventHandler("PlayerCurrencyChanged", "Redraw", self)
    Apollo.RegisterEventHandler("AccountCurrencyChanged", "Redraw", self)
    -- Register for ready signal of window position management
    Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)

    -- Timer to wait for game to be ready before loading 
    self.timerInit = ApolloTimer.Create(1, false, "OnInitDelay", self)
  end
end

function CurrencyCount:OnWindowManagementReady()
  -- If I ever need to reset window positioning, increment nSaveVersion
  Event_FireGenericEvent("WindowManagementRegister", {strName = "Currency Count", nSaveVersion=1})
  Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Currency Count", nSaveVersion=1})
  -- Redraw the window if WindowManamentReady happens after OnInitDelay to update position
  if self.timerInit == nil then self:Redraw() end
end

-----------------------------------------------------------------------------------------------
-- CurrencyCount Functions
-----------------------------------------------------------------------------------------------

function CurrencyCount:OnInitDelay()
  -- check if player is loaded in
  if GameLib.GetPlayerUnit() then
    -- Draw the UI and stop the timer
    self:Redraw()
    self.timerInit = nil
  else
    -- Restart timer and wait some more
    self.timerInit = ApolloTimer.Create(1, false, "OnInitDelay", self)
    return
  end
end

function CurrencyCount:Redraw()
  -- Delete any existing items
  self.wndMain:DestroyChildren()

  self.nCurrentWidth = 0
  local nLeft, nTop, nRight, nBottom = self.wndMain:GetAnchorOffsets()
  local nNewWidth = self.nMargin

  -- If enabled draw the background
  if self.tOptions.bEnableBackground then Apollo.LoadForm(self.xmlDoc, "CurrencyListBG", self.wndMain, self) end

  -- Check each currency, draw them and determine total width
  if self.tOptions.bShowCredits then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(1) end
  if self.tOptions.bShowRenown then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(2) end
  if self.tOptions.bShowPrestige then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(5) end
  if self.tOptions.bShowElder then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(3) end
  if self.tOptions.bShowCrafting then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(6) end
  if self.tOptions.bShowGlory then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(4) end
  if self.tOptions.bShowOmni then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(7) end
  if self.tOptions.bShowService then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(8) end
  if self.tOptions.bShowFortune then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(9) end
  if self.tOptions.bShowNCoin then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(10) end
  if self.tOptions.bShowPromissory then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(11) end
  if self.tOptions.bShowTriploons then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(12) end
  if self.tOptions.bShowViolet then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(13) end
  if self.tOptions.bShowCrimson then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(14) end
  if self.tOptions.bShowCobalt then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(15) end
  if self.tOptions.bShowViridian then nNewWidth = nNewWidth + self.nMargin + self:DrawCurrency(16) end

  -- Resize the main window to fit all the currency displays
  self.wndMain:SetAnchorOffsets((nRight-nNewWidth),nTop,nRight,nBottom)
end

function CurrencyCount:DrawCurrency(idx)
  -- Create the currency display
  local tData = self.karCurrency[idx]
  local wnd = Apollo.LoadForm(self.xmlDoc, "CurrencyItem", self.wndMain, self)
  local wndChild = wnd:FindChild("CurrencyDisplay")
  
  -- Set display to the correct currency and update the amount of the currency
  -- Account and player currencies need to be handled separately
  if tData.bAccountItem then
     wndChild:SetMoneySystem(Money.CodeEnumCurrencyType.GroupCurrency, 0, 0, tData.eType)
     wndChild:SetAmount(AccountItemLib.GetAccountCurrency(tData.eType), true)
  else
    wndChild:SetMoneySystem(tData.eType)
    wndChild:SetAmount(GameLib.GetPlayerCurrency(tData.eType), true)
  end
  
  -- Set the tooltip, and add cap info to omni and elder
  local strTooltip = tData.strTitle
  
  if tData.eType == AccountItemLib.CodeEnumAccountCurrency.Omnibits then
    local tOmniBitInfo = GameLib.GetOmnibitsBonusInfo()
    strTooltip = strTooltip.."\nEarned "..tOmniBitInfo.nWeeklyBonusEarned.."/"..tOmniBitInfo.nWeeklyBonusMax.." ("..(tOmniBitInfo.nWeeklyBonusMax-tOmniBitInfo.nWeeklyBonusEarned)..")"
  end

  if tData.eType == Money.CodeEnumCurrencyType.ElderGems then
    --GetPeriodicElderPoints() is a global function that returns total Elder Points earned this week, 
    --  divide by Elder Points per Gem to get current Gems earned
    local nCurrentEG = math.floor(GetPeriodicElderPoints() / GameLib.ElderPointsPerGem)
    -- GameLib.ElderPointsDailyMax returns the weekly EP cap. Used to be daily hence the name.
    local nWeeklyEg = math.floor(GameLib.ElderPointsDailyMax / GameLib.ElderPointsPerGem)

    strTooltip = strTooltip .. "\nEarned "..nCurrentEG .."/"..nWeeklyEg.." ("..(nWeeklyEg-nCurrentEG)..")"
  end

  wndChild:SetTooltip(strTooltip)

  -- Link this window to the currency idx, so that it can be identified later
  self.tCurrencyDisplays[idx]=wndChild:GetId()

  -- Reposition the display left from the currently drawn currencies
  local nLeft, nTop, nRight, nBottom = wnd:GetAnchorOffsets()
  local nWidth = 0

  -- Check if width is static or dynamic
  if self.tOptions.bSaticWidth then
    nWidth = self.tOptions.nSetWidth
    -- if it's displaying gold (idx 1) then make bigger
    if idx == 1 then nWidth = nWidth * 2 end
  else
    nWidth = wndChild:GetDisplayWidth()
  end

  self.nCurrentWidth = self.nCurrentWidth - self.nMargin
  wnd:SetAnchorOffsets(self.nCurrentWidth - nWidth, nTop, self.nCurrentWidth, nBottom)
  self.nCurrentWidth = self.nCurrentWidth - nWidth

  return nWidth
end

-----------------------------------------------------------------------------------------------
-- CurrencyCount Options Functions
-----------------------------------------------------------------------------------------------

function CurrencyCount:OnConfigure()
  if not self.wndOptions then
    self.wndOptions = Apollo.LoadForm(self.xmlDoc, "CurrencyOptions", nil, self)
  end
  self.wndOptions:Show(true,false)
  self.wndOptions:FindChild("CraftingButton"):SetCheck(self.tOptions.bShowCrafting)
  self.wndOptions:FindChild("RenownButton"):SetCheck(self.tOptions.bShowRenown)
  self.wndOptions:FindChild("PrestigeButton"):SetCheck(self.tOptions.bShowPrestige)
  self.wndOptions:FindChild("ElderButton"):SetCheck(self.tOptions.bShowElder)
  self.wndOptions:FindChild("GloryButton"):SetCheck(self.tOptions.bShowGlory)
  self.wndOptions:FindChild("OmniButton"):SetCheck(self.tOptions.bShowOmni)
  self.wndOptions:FindChild("PromissoryNotesButton"):SetCheck(self.tOptions.bShowPromissory)
  self.wndOptions:FindChild("TriploonsButton"):SetCheck(self.tOptions.bShowTriploons)
  self.wndOptions:FindChild("VioletButton"):SetCheck(self.tOptions.bShowViolet)
  self.wndOptions:FindChild("CrimsonButton"):SetCheck(self.tOptions.bShowCrimson)
  self.wndOptions:FindChild("CobaltButton"):SetCheck(self.tOptions.bShowCobalt)
  self.wndOptions:FindChild("ViridianButton"):SetCheck(self.tOptions.bShowViridian)
  self.wndOptions:FindChild("ServiceButton"):SetCheck(self.tOptions.bShowService)
  self.wndOptions:FindChild("FortuneButton"):SetCheck(self.tOptions.bShowFortune)
  self.wndOptions:FindChild("NCoinButton"):SetCheck(self.tOptions.bShowFortune)
  self.wndOptions:FindChild("CreditsButton"):SetCheck(self.tOptions.bShowCredits)
  self.wndOptions:FindChild("StaticWidthButton"):SetCheck(self.tOptions.bSaticWidth)
  self.wndOptions:FindChild("SetWidthBlocker"):Show(not self.tOptions.bSaticWidth)
  self.wndOptions:FindChild("SetWidthSlider"):SetValue(self.tOptions.nSetWidth)
  self.wndOptions:FindChild("SetWidthEditBox"):SetText(self.tOptions.nSetWidth)
  self.wndOptions:FindChild("BackgroundButton"):SetCheck(self.tOptions.bEnableBackground)  
end

function CurrencyCount:OnOptionsChange()
  self.tOptions.bShowRenown = self.wndOptions:FindChild("RenownButton"):IsChecked()
  self.tOptions.bShowPrestige = self.wndOptions:FindChild("PrestigeButton"):IsChecked()
  self.tOptions.bShowElder = self.wndOptions:FindChild("ElderButton"):IsChecked()
  self.tOptions.bShowCrafting = self.wndOptions:FindChild("CraftingButton"):IsChecked()
  self.tOptions.bShowCredits = self.wndOptions:FindChild("CreditsButton"):IsChecked()
  self.tOptions.bShowGlory = self.wndOptions:FindChild("GloryButton"):IsChecked()
  self.tOptions.bShowPromissory = self.wndOptions:FindChild("PromissoryNotesButton"):IsChecked()
  self.tOptions.bShowTriploons = self.wndOptions:FindChild("TriploonsButton"):IsChecked()
  self.tOptions.bShowViolet = self.wndOptions:FindChild("VioletButton"):IsChecked()
  self.tOptions.bShowCrimson = self.wndOptions:FindChild("CrimsonButton"):IsChecked()
  self.tOptions.bShowCobalt = self.wndOptions:FindChild("CobaltButton"):IsChecked()
  self.tOptions.bShowViridian = self.wndOptions:FindChild("ViridianButton"):IsChecked()
  self.tOptions.bShowOmni = self.wndOptions:FindChild("OmniButton"):IsChecked()
  self.tOptions.bShowService = self.wndOptions:FindChild("ServiceButton"):IsChecked()
  self.tOptions.bShowFortune = self.wndOptions:FindChild("FortuneButton"):IsChecked()
  self.tOptions.bShowNCoin = self.wndOptions:FindChild("NCoinButton"):IsChecked()
  self.tOptions.bSaticWidth = self.wndOptions:FindChild("StaticWidthButton"):IsChecked()
  self.tOptions.nSetWidth = self.wndOptions:FindChild("SetWidthSlider"):GetValue()
  self.tOptions.bEnableBackground = self.wndOptions:FindChild("BackgroundButton"):IsChecked()
  self.wndOptions:FindChild("SetWidthBlocker"):Show(not self.tOptions.bSaticWidth)
  self.wndOptions:FindChild("SetWidthEditBox"):SetText(self.tOptions.nSetWidth)
  
  self:Redraw()
end

function CurrencyCount:OnOptionsClose()
  self.wndOptions:Show(false,false)
end

-----------------------------------------------------------------------------------------------
-- CurrencyCount Instance
-----------------------------------------------------------------------------------------------
local CurrencyCountInst = CurrencyCount:new()
CurrencyCountInst:Init()
